<?php

namespace Lc\ShopBundle\Form;

use App\Entity\Supplier;
use Lc\ShopBundle\Context\GroupUserInterface;
use Lc\ShopBundle\Context\ProductCategoryInterface;
use Lc\ShopBundle\Context\ProductFamilyInterface;
use Lc\ShopBundle\Context\ReductionCatalogInterface;
use Lc\ShopBundle\Context\UserInterface;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\DateTimeType;
use Symfony\Component\Form\Extension\Core\Type\NumberType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;


class ReductionCatalogType extends AbstractType
{
        protected $em;

        public function __construct(EntityManagerInterface $entityManager)
        {
                $this->em = $entityManager;
        }

        public function buildForm(FormBuilderInterface $builder, array $options)
        {

                $userClass = $this->em->getClassMetadata(UserInterface::class);
                $groupUserClass = $this->em->getClassMetadata(GroupUserInterface::class);
                $supplierClass = Supplier::class;
                $productFamilyClass = $this->em->getClassMetadata(ProductFamilyInterface::class);
                $productCategoryClass = $this->em->getClassMetadata(ProductCategoryInterface::class);


                $builder
                        ->add('title', TextType::class, [
                                'label' => 'Titre',
                                'required' => false,
                        ])
                        ->add('behaviorTaxRate', ChoiceType::class, [
                                'required' => false,
                                'choices' => [
                                        'field.default.taxIncluded' => 'tax-included',
                                        'field.default.taxExcluded' => 'tax-excluded'
                                ]
                        ])
                        ->add('unit', ChoiceType::class, [
                                'required' => false,
                                'choices' => [
                                        'field.default.percent' => 'percent',
                                        'field.default.amount' => 'amount'
                                ]
                        ])
                        ->add('value', NumberType::class, [
                                'required' => false
                        ])
                        ->add('permanent', CheckboxType::class, array(
                                'required'=>false
                        ))
                        ->add('dateStart', DateTimeType::class,
                                ['widget' => 'single_text',
                                'required'=>false,
                                        ])
                        ->add('dateEnd', DateTimeType::class, [
                                'widget' => 'single_text',
                                'required'=>false,
                        ])
                        ->add('usersActive', CheckboxType::class, array(
                                'label' => 'field.ReductionCatalog.usersActive',
                                'mapped' => false,
                                'required'=>false,
                                'attr' => ['class' => 'big']
                        ))
                        ->add('users', EntityType::class, array(
                                'class' => $userClass->name,
                                'required'=>false,
                                'multiple'=>true
                        ))
                        ->add('groupUsersActive', CheckboxType::class, array(
                                'label' => 'field.ReductionCatalog.groupUsersActive',
                                'mapped' => false,
                                'required'=>false,
                                'attr' => ['class' => 'big']
                        ))
                        ->add('groupUsers', EntityType::class, array(
                                'class' => $groupUserClass->name,
                                'required'=>false,
                                'multiple'=>true

                        ));
                       /* ->add('productFamiliesActive', CheckboxType::class, array(
                                'label' => 'field.ReductionCatalog.productFamiliesActive',
                                'mapped' => false,
                                'required'=>false,
                                'attr' => ['class' => 'big']
                        ))
                        ->add('productFamilies', EntityType::class, array(
                                'class' => $productFamilyClass->name,
                                'required'=>false,
                                'multiple'=>true
                        ))
                        ->add('productCategoriesActive', CheckboxType::class, array(
                                'label' => 'field.ReductionCatalog.productCategoriesActive',
                                'mapped' => false,
                                'required'=>false,
                                'attr' => ['class' => 'big']
                        ))
                        ->add('productCategories', EntityType::class, array(
                                'class' => $productCategoryClass->name,
                                'required'=>false,
                                'multiple'=>true
                        ))
                        ->add('suppliersActive', CheckboxType::class, array(
                                'label' => 'field.ReductionCatalog.suppliersActive',
                                'mapped' => false,
                                'required'=>false,
                                'attr' => ['class' => 'big']
                        ))
                        ->add('suppliers', EntityType::class, array(
                                'class' => $supplierClass,
                                'required'=>false,
                                'multiple'=>true
                        ));*/

        }

        public function configureOptions(OptionsResolver $resolver)
        {
                $resolver->setDefaults([
                        'label' => false,
                        //'data_class' => $this->em->getClassMetadata(ReductionCatalogInterface::class)->getName(),
                        'translation_domain'=> 'lcshop'
                ]);
        }
}
